package dk.apaq.framework.repository.mongo;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import dk.apaq.framework.criteria.mongo.CriteriaForMongoDb;
import dk.apaq.framework.repository.BaseRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.mongodb.core.MongoOperations;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;

/**
 *
 * @author krog
 */
public class MongodbRepository<BEANTYPE, IDTYPE> extends BaseRepository<BEANTYPE, IDTYPE> {
    
    private static final Logger LOG = LoggerFactory.getLogger(MongodbRepository.class);
    private final Class<BEANTYPE> clazz;
    private MongoOperations mongoOperations;
    
    public MongodbRepository(MongoOperations mongoOperations, Class<BEANTYPE> clazz) {
        this.mongoOperations = mongoOperations;
        this.clazz = clazz;
        
        if (clazz == null) {
            throw new NullPointerException("Class was null");
        }

        //TODO Make sure the clazz has an _id field
    }

    public IDTYPE getIdFromEntity(BEANTYPE entity) {
        return resolveId(entity);
    }
    
    public BEANTYPE findOne(IDTYPE id) {
        fireOnBeforeRead(id);
        BEANTYPE entity = mongoOperations.findById(id, clazz);
        fireOnRead(id, entity);
        return entity;
    }
    
    public List<IDTYPE> findAllIds(dk.apaq.framework.criteria.Criteria criteria) {
        fireOnBeforeList(criteria);
        Query q = CriteriaForMongoDb.translate(new String[]{"id"}, clazz, criteria);
        List<BEANTYPE> entities = mongoOperations.find(q, clazz);
        
        List<IDTYPE> idlist = new ArrayList<IDTYPE>();
        for (BEANTYPE entity : entities) {
            idlist.add(resolveId(entity));
        }
        return idlist;
    }
    
    public List<BEANTYPE> findAll(dk.apaq.framework.criteria.Criteria criteria) {
        fireOnBeforeList(criteria);
        Query q = CriteriaForMongoDb.translate(clazz, criteria);
        return mongoOperations.find(q, clazz);
    }
    
    public BEANTYPE save(BEANTYPE entity) {
        IDTYPE id = resolveId(entity);
        if (id == null) {
            fireOnBeforeCreate(entity);
            mongoOperations.save(entity);
            fireOnCreate(resolveId(entity), entity);
            return entity;
        } else {
            fireOnBeforeUpdate(id, entity);
            mongoOperations.save(entity);
            fireOnUpdate(id, entity);
            return entity;
        }
    }

    public void delete(BEANTYPE entity) {
        deleteById(resolveId(entity));
    }
    
    public void deleteById(IDTYPE id) {
        fireOnBeforeDelete(id);
        mongoOperations.remove(new Query(Criteria.where("_id").is(id)), clazz);
        fireOnDelete(id);
    }
    
    private IDTYPE resolveId(BEANTYPE entity) {
        try {
            //MongoDb always has ids as field '_id';
            Field field = entity.getClass().getDeclaredField("_id");
            return (IDTYPE) field.get(entity);
        } catch (Exception ex) {
            LOG.warn("Unable to retrieve id for entity.", ex);
            return null;
        }
    }
}
